
/**
 This routine transforms an image using 3x3 matrix. The input
 matrix is "im_in" and the transformated image will be "im_out".
 All the interpolations are bilinears.

 matrix: matrix for transform the input image
 [x' y' 1] = | a d 0 | [x y 1]  ecuation
             | b e 0 | 
             | c f 1 |
**/

void transform (float **im_in, float **im_out, int row, int col, 
                double matrix[3][3])
{
	int i, j, k, l, row1, col1;
	float xprime, yprime, cen_row, cen_col;
	float val1, val2, scalex, scaley, y, x, scale1;
	float mat00, mat01;

	cen_row = row/2.0f - 0.5;  /**** este es el centro ****/
	cen_col = col/2.0f - 0.5;

	row1 = row-1;
	col1 = col-1;
	mat00 = matrix[0][0];
	mat01 = matrix[0][1];

	for (i=0; i < row; i++)       /** Clean output image **/
		for (j=0; j < col; j++)
			im_out[i][j] = 0.0f;

	for (i=0; i < row; i++) {
		y = i - cen_row;
		x = -cen_col;
		/** inicial point **/
		xprime = x*matrix[0][0] + y*matrix[1][0] + matrix[2][0] + cen_col;
		yprime = x*matrix[0][1] + y*matrix[1][1] + matrix[2][1] + cen_row;
		for (j = 0; j < col; j++) {   /**new point is xprime, yprime **/
			/** Now, to interpolate **/
			k = (int)yprime;
			l = (int)xprime;
			if (k < 0 || l < 0 || k >= row1 || l >= col1)
				im_out[i][j] = 0;
			else {
				scalex = xprime - l;
				scaley = yprime - k;
				scale1 = 1.0f - scalex;
				val1 = scalex*im_in[k][l+1]   + scale1*im_in[k][l];
				val2 = scalex*im_in[k+1][l+1] + scale1*im_in[k+1][l];
				im_out[i][j] = scaley * val2 + (1.0f-scaley) * val1;
			}
			xprime += mat00; 
			yprime += mat01;
		}
	}
}

